/*
	PROforma, PROgs FOnt & Raster MAnager

	Black drawing routines for mode 33 bitmap driver

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	started 9 November 1999
	21.10.2000 fixed small problem (only 7 of 8 bits in mask used)
	05.11.2000 - moved all DrawMask routines in this file, using similarities
			   - fixed a small clipping problem, too many bits are drawn when
				 clipping on the left, and starting too early in mask
			   - modification for drawing anti-aliased fonts, this implies
				 correcting mask->x/ysiz as the char is shown at half size.

*/

#include "PFmodule.h"

static const unsigned char bitmasks[]={ 0xc0, 0x30, 0x0c, 0x03 };

/*static const unsigned char bitmasks[]={ 0x80, 0x40, 0x20, 0x10, 8,4,2,1};*/


static const unsigned char bitcount[]={
	0,1, /* 1 */
	1,2, /* 3 */
	1,2,2,3, /* 7 */
	1,2,2,3,2,3,3,4, /* 15 */
	1,2,2,3,2,3,3,4,2,3,3,4,3,4,4,5, /* 31 */
	1,2,2,3,2,3,3,4,2,3,3,4,3,4,4,5,2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6, /* 63 */
	1,2,2,3,2,3,3,4,2,3,3,4,3,4,4,5,2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,
	2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,3,4,4,5,4,5,5,6,4,5,5,6,5,6,6,7, /* 127 */
	1,2,2,3,2,3,3,4,2,3,3,4,3,4,4,5,2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,
	2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,3,4,4,5,4,5,5,6,4,5,5,6,5,6,6,7,
	2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,3,4,4,5,4,5,5,6,4,5,5,6,5,6,6,7,
	3,4,4,5,4,5,5,6,4,5,5,6,5,6,6,7,4,5,5,6,5,6,6,7,5,6,6,7,6,7,7,8  /* 255 */
/* logica - per lijn alle waarden voor extra bit */
/* vanaf derde lijn, herhaal vorige lijn gevolg door waarden vorige lijn + 1 */
};

/* ----------- definitions for the code */

#define declarations														\
	short i,j;						/* loop counter */						\
	unsigned char *rlinestart;		/* curr pos in raster */				\
	unsigned short *rbase;			/* curr pos in bitmap */				\
	unsigned char *mbase, *mlinestart;	/* curr pos in mask */				\
	short linecount;				/* number of bytes to check in mask */	\
	unsigned char data; 			/* current part of char */				\
	int nrlines;															\
	int first=0, last, count;		/* first & last bit to display */		\
	short mlinc=mask->linc; 		/* line length in mask */				\
	short colred, colgreen, colblue;/* drawing colours */					\
	unsigned char intensity 		/* intensity for pixel */
	

#define initialise															\
	mlinestart=mask->base;													\
	linecount=mask->linc;													\
	nrlines=mask->ysiz; 													\
	last=mask->xsiz

#define clip_horizontal 													\
	/* horizontal clipping */												\
	if (x<0 || mask->xsiz+x>bitmap->clip.xsiz)								\
	{                                                                       \
		/* still ok if completely invisible */								\
		if (x+mask->xsiz<=0 || x>=bitmap->clip.xsiz)						\
			return ERR_OK;													\
		else																\
		{                                                                   \
			/* the actual clipping */										\
			if (x<0)														\
			{                                                               \
				last+=x;													\
				first=-((-x)&3);											\
				x=(-x)/4;													\
				mlinestart+=x;												\
				linecount-=x;												\
				x=0;														\
			}																\
			if (last+x>bitmap->clip.xsiz)									\
				last=bitmap->clip.xsiz-x;									\
		}																	\
	}

#define clip_vertical														\
	/* vertical clipping */ 												\
	if (y<0) { nrlines+=y; mlinestart+=(-y)*mask->linc*2; y=0; }            \
	if (nrlines<=0) return ERR_OK;											\
	if (nrlines+y > bitmap->clip.ysiz) nrlines-=nrlines+y-bitmap->clip.ysiz;\
	if (nrlines<=0) return ERR_OK

#define loop_start															\
	x+=bitmap->clip.xorg;													\
	y+=bitmap->clip.yorg;													\
	rlinestart=bitmap->base+y*bitmap->linc+x*2; 							\
	for( ; nrlines ; nrlines--) 											\
	{                                                                       \
		rbase=(unsigned short *)rlinestart; 								\
		mbase=mlinestart;													\
		count=first;														\
		for (i=linecount ; i ; i--) 										\
		{                                                                   \
			unsigned char *bits=bitmasks;									\
			for (j=4 ; j && count<=last ; j--)								\
			{                                                               \
				if (count>=0) {                                             \
					intensity =bitcount[mbase[0    ]&*bits]+				\
							   bitcount[mbase[mlinc]&*bits]

#define loop_end															\
					rbase++;												\
				}															\
				bits++; 													\
				count++;													\
			}																\
			*mbase++;														\
		}																	\
		rlinestart+=bitmap->linc;											\
		mlinestart+=mlinc+mlinc;											\
	}																		\
	return ERR_OK

/* ----------- actual anti-aliasing */

#define split32(c)															\
	colblue=c;																\
	colgreen=((colblue>>5)&0x3f);											\
	colred=((colblue>>11)&0x1f);											\
	colblue&=0x1f

#define anti_alias32(r,c,i) {                                               \
	if (i==4) *r=c; else if (i>0) {                                         \
		/* extract background colour */ 									\
		short blue=*r;														\
		short red=((blue>>11)&0x1f);										\
		short green=((blue>>5)&0x3f);										\
		blue&=0x1f; 														\
		/* calculate new colours */ 										\
		blue=blue+(colblue-blue)*i/4;										\
		red=red+(colred-red)*i/4;											\
		green=green+(colgreen-green)*i/4;									\
		/* set */															\
		if (red  >0x1f) red  =0x1f; 										\
		if (green>0x3f) green=0x3f; 										\
		if (blue >0x1f) blue =0x1f; 										\
		red  &=0x1f;														\
		green&=0x3f;														\
		blue &=0x1f;														\
		*r=blue + (red<<11) + (green<<5);									\
}}

#define split33(c)															\
	colblue=c;																\
	colred=((colblue>>5)&0x3e) + (colblue&1);								\
	colgreen=((colblue>>10)&0x3e) + (colblue&1);							\
	colblue&=0x3f

#define anti_alias33(r,c,i) {                                               \
	if (i==4) *r=c; else if (i>0) {                                         \
		/* extract background colour */ 									\
		short blue=*r;														\
		short red=((blue>>5)&0x3e) + (blue&1);									\
		short green=((blue>>10)&0x3e) + (blue&1);								 \
		blue&=0x3f; 														\
		/* calculate new colours */ 										\
		blue=blue+(colblue-blue)*i/4;										\
		red=red+(colred-red)*i/4;											\
		green=green+(colgreen-green)*i/4;									\
		/* set */															\
		if (red  >0x3f) red  =0x3f; 										\
		if (green>0x3f) green=0x3f; 										\
		if (blue >0x3f) blue =0x3f; 										\
		if ((red&1)+(green&1)+(blue&1)>1) blue|=1; else blue&=0x3e; 		\
		red  &=0x3e;														\
		green&=0x3e;														\
		blue &=0x3f;														\
		*r=blue + (red<<5) + (green<<10);									\
}}


/* ----------- actual routines */

Error BlackDrawMask32(Bitmap bitmap, Mask *mask, short x, short y)
{
	declarations;
	clip_horizontal;
	clip_vertical;
	colred=colgreen=colblue=0;
	loop_start;
		anti_alias32(rbase,0,intensity);
	loop_end;
}

Error WhiteDrawMask32(Bitmap bitmap, Mask *mask, short x, short y)
{
	declarations;
	clip_horizontal;
	clip_vertical;
	colred=colgreen=colblue=0x3f;
	loop_start;
		anti_alias32(rbase,0xffff,intensity);
	loop_end;
}

Error FastPatternDrawMask32(Bitmap bitmap, Mask *mask, short x, short y)
{
	declarations;
	int colour=*((unsigned short *)bitmap->pattern);
	initialise;
	clip_horizontal;
	clip_vertical;
	split32(colour);
	loop_start;
		anti_alias32(rbase,colour,intensity);
	loop_end;
}

Error BlackDrawMask33(Bitmap bitmap, Mask *mask, short x, short y)
{
	declarations;
	clip_horizontal;
	clip_vertical;
	colred=colgreen=colblue=0;
	loop_start;
		anti_alias33(rbase,0,intensity);
	loop_end;
}

Error WhiteDrawMask33(Bitmap bitmap, Mask *mask, short x, short y)
{
	declarations;
	clip_horizontal;
	clip_vertical;
	colred=colgreen=colblue=0x3f;
	loop_start;
		anti_alias33(rbase,0xffff,intensity);
	loop_end;
}

Error FastPatternDrawMask33(Bitmap bitmap, Mask *mask, short x, short y)
{
	declarations;
	int colour=*((unsigned short *)bitmap->pattern);
	initialise;
	clip_horizontal;
	clip_vertical;
	split33(colour);
	loop_start;
		anti_alias33(rbase,colour,intensity);
	loop_end;
}
