/* PROforma, PROgs FOnt & Raster MAnager

	BITMdriver definition file for screen driver

   (c) PROGS, Professional & Graphical Software
   written by Joachim Van der Auwera
   started 9 November 1999
   05.11.2000 modified for both mode 33 and mode 32 with anti-aliased fonts
   */

#include <win.h>
#include "PFmodule.h"

static Error nimp()    { return ERR_NIMP; }
static Error nothing() { return ERR_OK;   }

Error _LineVertical(Bitmap, short atx, short fromy, short toy);
Error _Block(Bitmap, short xsiz, short ysiz, short xorg, short yorg);
Error _TrapezoidHorizontal(Bitmap, short starty, short endy,
				pt leftstartx, pt leftincr, pt rightstartx, pt rightincr);
Error _TrapezoidVertical(Bitmap, short startx, short endx,
				pt topstarty, pt topincr, pt bottomstarty, pt bottomincr);
Error _HairLine(Bitmap, pt fromx, pt fromy, pt tox, pt toy);

Error BlackPixel(Bitmap, short x, short y);
Error BlackLineHorizontal(Bitmap, short aty, short fromx, short tox);
Error BlackDrawMask32(Bitmap, Mask *mask, short x, short y);
Error BlackDrawMask33(Bitmap, Mask *mask, short x, short y);

Error WhitePixel(Bitmap, short x, short y);
Error WhiteLineHorizontal(Bitmap, short aty, short fromx, short tox);
Error WhiteDrawMask32(Bitmap, Mask *mask, short x, short y);
Error WhiteDrawMask33(Bitmap, Mask *mask, short x, short y);

Error FastPatternPixel(Bitmap, short x, short y);
Error FastPatternLineHorizontal(Bitmap, short aty, short fromx, short tox);
Error FastPatternDrawMask32(Bitmap, Mask *mask, short x, short y);
Error FastPatternDrawMask33(Bitmap, Mask *mask, short x, short y);
Error FastPatternChangeOrigin(Bitmap, short x, short y);

Error PatternPixel(Bitmap, short x, short y);
Error PatternLineHorizontal(Bitmap, short aty, short fromx, short tox);
Error PatternDrawMask(Bitmap, Mask *mask, short x, short y);
Error PatternChangeOrigin(Bitmap, short x, short y);

Error CursorSet(Bitmap, short x, short y);
Error CursorMove(Bitmap, short x, short y);
Error CursorGet(Bitmap, short *x, short *y);

Error PixelRead(Bitmap, int *colour);
Error PixelReadNextX(Bitmap, int *colour);
Error PixelReadPrevX(Bitmap, int *colour);
Error PixelReadNextY(Bitmap, int *colour);
Error PixelReadPrevY(Bitmap, int *colour);
Error PixelSet(Bitmap, int colour);
Error PixelSetNextX(Bitmap, int colour);
Error PixelSetPrevX(Bitmap, int colour);
Error PixelSetNextY(Bitmap, int colour);
Error PixelSetPrevY(Bitmap, int colour);

Error CopyArea(Bitmap from, Bitmap to, short xsiz, short ysiz,
			   short fromx, short fromy, short tox, short toy);


Error BufferSize(short xsiz, short ysiz, Size *size)
{
	*size=xsiz*2 * ysiz;
	return ERR_OK;
}


BITMdriver mode32={
	NULL, PF_BITMAPDRIVER,
	{
		1, 16,					/* 3 planes, each 5 pixel deep */
		WINDOWMODE_16BITQPC,	/* bytegrouping used for bit mapping in byte */
		BITMAP_NO_PLANES		/* altenative bytes in each plane */
	},

	/* drawing routines */
	{                               /* Black drawing routines */
		BlackPixel,
		BlackLineHorizontal,
		_LineVertical,
		_Block,
		_TrapezoidHorizontal,
		_TrapezoidVertical,
		_HairLine,
		BlackDrawMask32,
		nothing
	},
	{                               /* White drawing routines */
		WhitePixel,
		WhiteLineHorizontal,
		_LineVertical,
		_Block,
		_TrapezoidHorizontal,
		_TrapezoidVertical,
		_HairLine,
		WhiteDrawMask32,
		nothing
	},
	{                               /* FastPattern drawing routines */
		FastPatternPixel,
		FastPatternLineHorizontal,
		_LineVertical,
		_Block,
		_TrapezoidHorizontal,
		_TrapezoidVertical,
		_HairLine,
		FastPatternDrawMask32,
		FastPatternChangeOrigin
	},
	{                               /* Pattern drawing routines */
		BlackPixel,
		BlackLineHorizontal,
		_LineVertical,
		_Block,
		_TrapezoidHorizontal,
		_TrapezoidVertical,
		_HairLine,
		BlackDrawMask32,
		nimp
	},

	CursorSet,
	CursorMove,
	CursorGet,

	PixelRead,
	PixelReadNextX,
	PixelReadPrevX,
	PixelReadNextY,
	PixelReadPrevY,
	PixelSet,
	PixelSetNextX,
	PixelSetPrevX,
	PixelSetNextY,
	PixelSetPrevY,

	CopyArea,

	BufferSize, /* BufferSize */
	8,8,		/* size of FastPattern */
	BITMAP_DIRECTION_VERTICAL,

	nimp
};

BITMdriver init={
	&mode32, PF_BITMAPDRIVER,
	{
		1, 16,					/* 3 planes, each 5 pixel deep */
		WINDOWMODE_16BITQ40,	/* bytegrouping used for bit mapping in byte */
		BITMAP_NO_PLANES		/* altenative bytes in each plane */
	},

	/* drawing routines */
	{                               /* Black drawing routines */
		BlackPixel,
		BlackLineHorizontal,
		_LineVertical,
		_Block,
		_TrapezoidHorizontal,
		_TrapezoidVertical,
		_HairLine,
		BlackDrawMask33,
		nothing
	},
	{                               /* White drawing routines */
		WhitePixel,
		WhiteLineHorizontal,
		_LineVertical,
		_Block,
		_TrapezoidHorizontal,
		_TrapezoidVertical,
		_HairLine,
		WhiteDrawMask33,
		nothing
	},
	{                               /* FastPattern drawing routines */
		FastPatternPixel,
		FastPatternLineHorizontal,
		_LineVertical,
		_Block,
		_TrapezoidHorizontal,
		_TrapezoidVertical,
		_HairLine,
		FastPatternDrawMask33,
		FastPatternChangeOrigin
	},
	{                               /* Pattern drawing routines */
		BlackPixel,
		BlackLineHorizontal,
		_LineVertical,
		_Block,
		_TrapezoidHorizontal,
		_TrapezoidVertical,
		_HairLine,
		BlackDrawMask33,
		nimp
	},

	CursorSet,
	CursorMove,
	CursorGet,

	PixelRead,
	PixelReadNextX,
	PixelReadPrevX,
	PixelReadNextY,
	PixelReadPrevY,
	PixelSet,
	PixelSetNextX,
	PixelSetPrevX,
	PixelSetNextY,
	PixelSetPrevY,

	CopyArea,

	BufferSize, /* BufferSize */
	8,8,		/* size of FastPattern */
	BITMAP_DIRECTION_VERTICAL,

	nimp
};
