/*
	PROforma, PROgs FOnt & Raster MAnager

	FastPattern drawing routines for mono bitmap driver

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	started 16 December 1995 based on mode4BlackDrawMask_c
*/

#include "PFmodule.h"

typedef unsigned int uint;
static Error FastPatternDrawMaskClipped(Bitmap bitmap, Mask *mask, short x, short y);

#define gooddivide(x,y) ( ((x)>=0) ? ((uint)(x))/(y) : -(((uint)(-(x)+7))/(y)) )

Error FastPatternDrawMask(Bitmap bitmap, Mask *mask, short x, short y)
{
	short i;						/* loop counter */
	unsigned char *rlinestart;		/* curr pos in raster */
	unsigned char *rbase, *mbase;	/* curr pos in bitmap / mask */
	unsigned char data; 			/* current part of char */
	unsigned char pattern;			/* FastPattern to use as colour */
	int yp; 						/* index in pattern */
	int rshift,lshift;
	int nrlines;

	/* perform clipping */
	if (x<0 || mask->xsiz+x>bitmap->clip.xsiz)
	{
		/* still ok if completely invisible */
		if (x+mask->xsiz<=0 || x>=bitmap->clip.xsiz)
			return ERR_OK;
		else
			return FastPatternDrawMaskClipped(bitmap,mask,x,y);
	}

	nrlines=mask->ysiz;
	mbase=mask->base;
	if (y<0) { nrlines+=y; mbase+=(-y)*mask->linc; y=0; }
	if (nrlines<=0) return ERR_OK;
	if (nrlines+y > bitmap->clip.ysiz) nrlines-=nrlines+y-bitmap->clip.ysiz;
	if (nrlines<=0) return ERR_OK;

	x+=bitmap->clip.xorg;
	y+=bitmap->clip.yorg;

	rlinestart=bitmap->base+y*bitmap->linc+x/8;
	yp=y&7;

	if (rshift=x&7)
	{
		lshift=8-rshift;
		for( ; nrlines ; nrlines--)
		{
			rbase=rlinestart;
			pattern=bitmap->pattern[yp];
			for (i=mask->linc;i;i--)
			{
				data=((*mbase  )>>rshift);
				*rbase	&=~data;
				*rbase++|=pattern&data;
				data=((*mbase++)<<lshift);
				*rbase	&=~data;
				*rbase	|=pattern&data;
			}
			rlinestart+=bitmap->linc;
			yp=(yp+1)&7;
		}
	}
	else
	{
		for( ; nrlines ; nrlines--)
		{
			rbase=rlinestart;
			pattern=bitmap->pattern[yp];
			for (i=mask->linc;i;i--)
			{
				data=(*mbase++);
				*rbase	&=~data;
				*rbase++|=pattern&data;
			}
			rlinestart+=bitmap->linc;
			yp=(yp+1)&7;
		}
	}
	return ERR_OK;
}

static Error FastPatternDrawMaskClipped(Bitmap bitmap, Mask *mask, short x, short y)
{
	short i;						/* loop counter */
	unsigned char *rbase, *mbase;	/* curr pos in bitmap / mask */
	unsigned char data; 			/* current part of char */
	unsigned char pattern;			/* FastPattern to use as colour */
	int yp; 						/* index in pattern */
	int rshift,lshift;
	int nrlines;
	int maskleft, maskright;
	int posleft,posright,posstart,pos;

	nrlines=mask->ysiz;
	mbase=mask->base;
	if (y<0)
	{
		nrlines+=y;
		if (nrlines<=0) return ERR_OK;
		mbase+=(-y)*mask->linc;
		y=0;
	}
	if (nrlines+y > bitmap->clip.ysiz)
	{
		nrlines-=nrlines+y-bitmap->clip.ysiz;
		if (nrlines<=0) return ERR_OK;
	}

	x+=bitmap->clip.xorg;
	y+=bitmap->clip.yorg;
	yp=y&7;
	rshift=x&7;
	rbase=bitmap->base+y*bitmap->linc;
	posleft =bitmap->clip.xorg;
	posright=bitmap->clip.xorg+bitmap->clip.xsiz;
	maskleft = (0xff >> (posleft &7));
	maskright=~(0xff >> (posright&7));
	posleft =gooddivide(posleft,8);
	posright=posright/8;
	posstart=gooddivide(x,8);

	if (rshift)
	{
		lshift=8-rshift;
		for( ; nrlines ; nrlines--)
		{
			pos=posstart;
			pattern=bitmap->pattern[yp];
			for (i=mask->linc ; i ; i--)
			{
				data=((*mbase  )>>rshift);
				if (pos==posright)
					data&=maskright;
				if (pos==posleft)
					data&=maskleft;
				if (pos>=posleft && pos<=posright)
				{
					*(rbase+pos)&=~data;
					*(rbase+pos)|=pattern&data;
				}

				pos++;
				data=((*mbase++)<<lshift);
				if (pos==posright)
					data&=maskright;
				if (pos==posleft)
					data&=maskleft;
				if (pos>=posleft && pos<=posright)
				{
					*(rbase+pos)&=~data;
					*(rbase+pos)|=pattern&data;
				}
			}
			rbase+=bitmap->linc;
			yp=(yp+1)&7;
		}
	}
	else
	{
		for( ; nrlines ; nrlines--)
		{
			pos=posstart;
			pattern=bitmap->pattern[yp];
			for (i=mask->linc ; i ; i--,pos++)
			{
				data=(*mbase++);
				if (pos==posright)
					data&=maskright;
				if (pos==posleft)
					data&=maskleft;
				if (pos>=posleft && pos<=posright)
				{
					*(rbase+pos)&=~data;
					*(rbase+pos)|=pattern&data;
				}
			}
			rbase+=bitmap->linc;
			yp=(yp+1)&7;
		}
	}
	return ERR_OK;
}
