/*
	PROforma, PROGS Font & Raster Manager

	mono pic printer driver, this will save the picture as a mono picture

	(c) PROGS, Professional & Graphical Software
	written by Joachim Van der Auwera
	04.12.1996 started based on bj-10 printer driver
*/

#include <io.h>
#include <mem.h>
#include <win.h>
#include "PFmodule.h"

#define RES 288

static Error Define(char *name, char *value)
{
	LinkPROforma();
	return ERR_IPAR;
}

static Error InitInit(Gstate gstate, char *device, PBbox *area)
{
	Error err;

	if (device && *device)
		catch( IOOpen(device,OPEN_NEW,(Channel *)&gstate->PrinterExtra) );
	else
		return ERR_ITNF;

	gstate->todev.x = short2pt(RES) / 72;
	gstate->todev.y = short2pt(RES) / 72;
	gstate->ppp.x = short2pt(72) / RES;
	gstate->ppp.y = short2pt(72) / RES;

	area->xsiz=RES*8.25;
	area->ysiz=RES*11.7;
	area->xorg=area->yorg=0;

	return err;
}

static Error InitDone(Gstate gstate)
{
	WindowArea area;
	PBbox bbx;

	/* write the file header */
	PFPPageBboxFullGet(gstate,&bbx);
	area.flag=WINDOWAREA_FLAG;
	area.xsiz=bbx.xsiz;
	area.ysiz=bbx.ysiz;
	area.line_increment=gstate->raster->linc;
	area.mode=WINDOWMODE_M4;
	area.zero=0;

	return IOPutRawBytes((Channel)gstate->PrinterExtra,
						 offsetof(WindowArea,data)-offsetof(WindowArea,flag),
						 (char *)&area.flag,NULL);
}

static Error BufferSize(Gstate gstate, int xsiz, int ysiz, Size *size)
{
	return gstate->bitmap->BufferSize(xsiz,ysiz,size);
}

static Error CleanUp(Gstate gstate)
{
	return IOClose((Channel)gstate->PrinterExtra);
}

Error ColourNearest(Gstate gstate, ColourRGB *colour, int *result)
{
	*result=0;						/* assume black */
	if (colour->green>pt_hundred/2)
		*result+=2; 				/* add some green */
	if (colour->red  >pt_hundred/2)
		*result+=1; 				/* add some red */
	return ERR_OK;
}

Error ColourDelta(Gstate gstate, ColourRGB *colour, ColourRGB *delta)
{
	MEMCopy(sizeof(ColourRGB),colour,delta);
	delta->blue=colour->blue;
	if (colour->green>pt_hundred/2)
		delta->green-=pt_hundred;
	if (colour->red  >pt_hundred/2)
		delta->red	-=pt_hundred;
	return ERR_OK;
}

/* !!!!!!!!!!! code to send data to printer !!!!!!!!!!! */

/* some macros for generating output */
#define printb(s,b) IOPutRawBytes(chid,s,b,NULL)

Error InitPage(Gstate gstate)
{
	return ERR_OK;
}

Error ShowPage(Gstate gstate)
{
	return IOPutRawBytes((Channel)gstate->PrinterExtra,
						 gstate->raster->linc*gstate->pagebbox.ysiz,
						 gstate->raster->base,NULL);
}

Error StopPage(Gstate gstate)
{
	return IOClose((Channel)gstate->PrinterExtra);
}


/* !!!!!!!!!!! printer driver definition !!!!!!!!!!! */

static Error nimp() 	{ return ERR_NIMP; }
static Error nothing()	{ return ERR_OK; }

Error InitDriver();

Error SetCopies(Gstate, int count);
Error SetDisplayMode(Gstate, int mode);
Error WindowMove(Gstate, int dx, int dy);
Error BufferUpdate(Gstate);
Error ColourAdjust(Gstate, ColourRGB *);
Error GetFullPageBbox(Gstate, Bbox *box);
Error DefaultDevice(Gstate, char *device);

#define InitDriver		nothing
#define WindowMove		nothing
#define ColourAdjust	(Error (*)(Gstate,ColourRGB *))nothing
#define SetDisplayMode	nimp
#define GetFullPageBbox nimp
#define DefaultDevice	nimp
#define SetCopies		nimp
#define BufferUpdate	(Error (*)(Gstate))nimp
#define BufferSave		(Error (*)(Gstate))nimp
#define BufferRestore	(Error (*)(Gstate))nimp
#define Handle			(Error (*)(int,...))nimp

DRIVdriver init = {
	NULL, PF_PRINTERDRIVER,
	"four colour _pic file",	/* name */
	RES, RES,					/* xdpi, ydpi */
	{                           /* mono bitmap */
		2, 1,					/* 1 plane, 1 pixel deep */
		BITMAP_NO_BYTEGROUPING, /* no byte grouping */
		1,						/* no planes */
	},

	InitDriver,
	Define,

	InitInit,
	BufferSize,
	InitDone,
	CleanUp,

	InitPage,
	ShowPage,
	StopPage,

	SetCopies,
	SetDisplayMode,
	WindowMove,

	BufferUpdate,
	BufferSave,
	BufferRestore,

	PF_COLOURSPACE_RGB,
	{
		ColourAdjust,
		ColourNearest,
		ColourDelta
	},

	GetFullPageBbox,
	DefaultDevice,

	Handle
};
